
import java.net.*;
import java.io.*;
import java.nio.*;
import java.util.regex.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.text.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.nio.channels.*;
import java.awt.image.*;
import javax.imageio.*;
import javax.net.*;
import javax.net.ssl.*;
import java.security.*;
import java.util.jar.*;
import java.security.cert.*;
import java.math.*;

public class GameSessionViewer extends JPanel {
    
    public static void main(String[] args) throws Exception {
        JFrame f = new JFrame("hello");
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.getContentPane().setLayout(new BorderLayout());
        f.getContentPane().add(new GameSessionViewer(new GameSession("mygame.xml")));
        
        f.pack();
        f.show();
    }
    
    public GameSession session;
    public Board board;
    public JSlider slider;
    public JTextArea annotation;
    public boolean showRect = false;
    public int rx, ry, rw, rh, symmetricSpace;
    public BoardPanel boardPanel;
    
    public GameSessionViewer(GameSession session) throws Exception {
        this.session = session;
        this.board = session.board;
        
        this.slider = new JSlider(0, session.interestingTimes.size() - 1, 0);
        slider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                updateAnnotation();
                boardPanel.repaint();
            }
        });
        boardPanel = new BoardPanel();
        annotation = new JTextArea();
        annotation.setFont(Font.decode("Monospaced"));
        annotation.setPreferredSize(new Dimension(200, 100));
        
        setLayout(new BorderLayout());
        JPanel p = new JPanel(new BorderLayout());
        JPanel pp = new JPanel();
        pp.add(boardPanel);
        p.add(pp);
        p.add(slider, BorderLayout.SOUTH);
        add(p, BorderLayout.WEST);
        add(annotation, BorderLayout.CENTER);
        
        updateAnnotation();
    }
    
    public void updateAnnotation() {
        showRect = false;
        try {
            String s = session.annotations.get(slider.getValue());
            Matcher m = Pattern.compile("rx = (.*?), ry = (.*?), rw = (.*?), rh = (.*?), s = (.*?)\n\n").matcher(s);
            if (m.find()) {
                showRect = true;
                rx = new Integer(m.group(1));
                ry = new Integer(m.group(2));
                rw = new Integer(m.group(3));
                rh = new Integer(m.group(4));
                symmetricSpace = new Integer(m.group(5));
                annotation.setText(s.substring(m.end(0), s.length()));
            } else {
                throw new Exception();
            }
        } catch (Exception e) {
            annotation.setText("");
        }
        if (slider.getValue() == slider.getMaximum()) {
            if (session.mistakes.size() > 0) {
                annotation.setText("game over because of a mistake");
            } else if (session.gameOver) {
                annotation.setText("mines swept!");
            } else {
                annotation.setText("...no more rules applied.");
            }
        }
    }
    
    public class BoardPanel extends JPanel {
        Map<String, BufferedImage> images = new HashMap<String, BufferedImage>();
        int tileWidth;
        int tileHeight;
        
        public BoardPanel() throws Exception {
            for (File f : new File("images").listFiles()) {
                images.put(u.trimExtension(f.getName()), ImageIO.read(f));
            }
            BufferedImage genericTile = images.get("1");
            tileWidth = genericTile.getWidth();
            tileHeight = genericTile.getHeight();
        }
    
        public Dimension getPreferredSize() {
            return new Dimension(board.width * tileWidth, board.height * tileHeight);
        }
        
        public String getSpotType(int x, int y) {
            Spot spot = new Spot(x, y);
            if (session.exploredBy(spot, session.interestingTimes.get(slider.getValue()))) {
                if (session.mistake(spot)) {
                    if (board.mine(spot)) {
                        return "red_mine";
                    } else {
                        return "x_mine";
                    }
                } else if (board.mine(spot)) {
                    return "mine";
                } else {
                    return "" + board.mineCount(spot);
                }
            } else {
                return "unexplored";
            }
        }
        
        public void paint(Graphics g1) {
            Graphics2D g = (Graphics2D)g1;
            
            for (int x = 0; x < board.width; x++) {
                for (int y = 0; y < board.height; y++) {
                    g.drawImage(images.get(getSpotType(x, y)), x * tileWidth, y * tileHeight, this);
                }
            }
            
            if (showRect) {
                BufferedImage up = images.get("up");
                AffineTransform t = new AffineTransform();
                t.translate(rx * tileWidth + (tileWidth / 2), ry * tileHeight + (tileWidth / 2));
                if ((symmetricSpace & 4) != 0) {
                    t.concatenate(new AffineTransform(0, 1, 1, 0, 0, 0));
                }
                if ((symmetricSpace & 1) != 0) {
                    t.concatenate(new AffineTransform(-1, 0, 0, 1, 0, 0));
                }
                if ((symmetricSpace & 2) != 0) {
                    t.concatenate(new AffineTransform(1, 0, 0, -1, 0, 0));
                }
                t.translate(-tileWidth / 2, -tileHeight / 2);
                t.scale((double)(rw * tileWidth) / up.getWidth(), (double)(rh * tileHeight) / up.getHeight());
                g.drawImage(up, t, this);
            }
        }
    }
}
